@@ -81,9 +81,12 @@ static Parse_choice * make_choice(Parse_set *lset, int llw, int lrw, Connector *
  */
 static void put_choice_in_set(Parse_set *s, Parse_choice *pc)
 {
-	if (s->first == NULL) {
+	if (s->first == NULL)
+	{
 		s->first = pc;
-	} else {
+	}
+	else 
+	{
 		s->current->next = pc;
 	}
 	s->current = pc;
@@ -117,6 +120,7 @@ void init_x_table(Sentence sent)
 
 	pi = sent->parse_info = (Parse_info) xalloc(sizeof(struct Parse_info_struct));
 	pi->N_words = sent->length;
+	pi->parse_set = NULL;
 
 	if (pi->N_words >= 10) {
 		x_table_size = (1<<14);
@@ -129,13 +133,14 @@ void init_x_table(Sentence sent)
 	/*printf("Allocating x_table of size %d\n", x_table_size);*/
 	pi->x_table_size = x_table_size;
 	pi->x_table = (X_table_connector**) xalloc(x_table_size * sizeof(X_table_connector*));
-	for (i=0; i<x_table_size; i++) {
+	for (i=0; i<x_table_size; i++)
+	{
 		pi->x_table[i] = NULL;
 	}
 }
 
 /**
- * This is the function that should be used to free tha set structure. Since
+ * This is the function that should be used to free the set structure. Since
  * it's a dag, a recursive free function won't work.  Every time we create
  * a set element, we put it in the hash table, so this is OK.
  */
@@ -144,13 +149,16 @@ static void free_x_table(Parse_info pi)
 	int i;
 	X_table_connector *t, *x;
 
-	if (pi->x_table == NULL) {
+	if (pi->x_table == NULL)
+	{
 		/*fprintf(stderr, "Warning: Tried to free a NULL x_table\n");*/
 		return;
 	}
 
-	for (i=0; i<pi->x_table_size; i++) {
-		for(t = pi->x_table[i]; t!= NULL; t=x) {
+	for (i=0; i<pi->x_table_size; i++)
+	{
+		for(t = pi->x_table[i]; t!= NULL; t=x)
+		{
 			x = t->next;
 			free_set(t->set);
 			xfree((void *) t, sizeof(X_table_connector));
@@ -221,7 +229,7 @@ static void x_table_update(int lw, int rw, Connector *le, Connector *re,
  * returns NULL if there are no ways to parse, or returns a pointer
  * to a set structure representing all the ways to parse.
  *
- * This code is similar to code in count.c 
+ * This code is similar to code in count.c
  * (grep for end_word in these files).
  */
 static Parse_set * parse_set(Sentence sent,
@@ -255,69 +263,78 @@ static Parse_set * parse_set(Sentence sent,
 
 	xt = x_table_pointer(lw, rw, le, re, cost, pi);
 
-	if (xt == NULL) {
-		xt = x_table_store(lw, rw, le, re, cost, empty_set(), pi);
-		/* start it out with the empty set of options */
-		/* this entry must be updated before we return */
-	} else {
-		return xt->set;  /* we've already computed it */
-	}
+	if (xt != NULL) return xt->set;  /* we've already computed it */
+
+	/* Start it out with the empty set of options. */
+	/* This entry must be updated before we return. */
+	xt = x_table_store(lw, rw, le, re, cost, empty_set(), pi);
 
 	xt->set->count = count;  /* the count we already computed */
 	/* this count is non-zero */
 
-	if (rw == 1+lw) return xt->set;
-	if ((le == NULL) && (re == NULL)) {
-		if (!islands_ok && (lw != -1)) {
-			return xt->set;
-		}
-		if (cost == 0) {
-			return xt->set;
-		} else {
-			w = lw+1;
-			for (dis = sent->word[w].d; dis != NULL; dis = dis->next) {
-				if (dis->left == NULL) {
-					rs[0] = parse_set(sent, dis, NULL, w, rw, dis->right, NULL, cost-1, islands_ok, pi);
-					if (rs[0] == NULL) continue;
-					a_choice = make_choice(dummy_set(), lw, w, NULL, NULL,
-										   rs[0], w, rw, NULL, NULL,
-										   NULL, NULL, NULL);
-					put_choice_in_set(xt->set, a_choice);
-				}
-			}
-			rs[0] = parse_set(sent, NULL, NULL, w, rw, NULL, NULL, cost-1, islands_ok, pi);
-			if (rs[0] != NULL) {
+	if (rw == 1 + lw) return xt->set;
+
+	if ((le == NULL) && (re == NULL))
+	{
+		if (!islands_ok && (lw != -1)) return xt->set;
+
+		if (cost == 0) return xt->set;
+
+		w = lw + 1;
+		for (dis = sent->word[w].d; dis != NULL; dis = dis->next)
+		{
+			if (dis->left == NULL)
+			{
+				rs[0] = parse_set(sent, dis, NULL, w, rw, dis->right,
+				                  NULL, cost-1, islands_ok, pi);
+				if (rs[0] == NULL) continue;
 				a_choice = make_choice(dummy_set(), lw, w, NULL, NULL,
 									   rs[0], w, rw, NULL, NULL,
 									   NULL, NULL, NULL);
 				put_choice_in_set(xt->set, a_choice);
 			}
-			return xt->set;
 		}
+		rs[0] = parse_set(sent, NULL, NULL, w, rw, NULL, NULL,
+		                  cost-1, islands_ok, pi);
+		if (rs[0] != NULL)
+		{
+			a_choice = make_choice(dummy_set(), lw, w, NULL, NULL,
+								   rs[0], w, rw, NULL, NULL,
+								   NULL, NULL, NULL);
+			put_choice_in_set(xt->set, a_choice);
+		}
+		return xt->set;
 	}
 
-	if (le == NULL) {
-		start_word = lw+1;
-	} else {
+	if (le == NULL)
+	{
+		start_word = lw + 1;
+	}
+	else
+	{
 		start_word = le->word;
-
 	}
 
-	if (re == NULL) {
+	if (re == NULL)
+	{
 		end_word = rw;
-	} else {
+	}
+	else
+	{
 		end_word = re->word + 1;
 	}
 
-	for (w=start_word; w < end_word; w++)
+	for (w = start_word; w < end_word; w++)
 	{
 		m1 = m = form_match_list(sent, w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next)
 		{
 			d = m->d;
-			for (lcost = 0; lcost <= cost; lcost++) {
+			for (lcost = 0; lcost <= cost; lcost++)
+			{
 				rcost = cost-lcost;
-				/* now lcost and rcost are the costs we're assigning to those parts respectively */
+				/* now lcost and rcost are the costs we're assigning to
+				 * those parts respectively */
 
 				/* Now, we determine if (based on table only) we can see that
 				   the current range is not parsable. */
@@ -325,23 +342,28 @@ static Parse_set * parse_set(Sentence sent,
 				Lmatch = (le != NULL) && (d->left != NULL) && do_match(sent, le, d->left, lw, w);
 				Rmatch = (d->right != NULL) && (re != NULL) && do_match(sent, d->right, re, w, rw);
 				for (i=0; i<4; i++) {ls[i] = rs[i] = NULL;}
-				if (Lmatch) {
+				if (Lmatch)
+				{
 					ls[0] = parse_set(sent, ld, d, lw, w, le->next, d->left->next, lcost, islands_ok, pi);
 					if (le->multi) ls[1] = parse_set(sent, ld, d, lw, w, le, d->left->next, lcost, islands_ok, pi);
 					if (d->left->multi) ls[2] = parse_set(sent, ld, d, lw, w, le->next, d->left, lcost, islands_ok, pi);
 					if (le->multi && d->left->multi) ls[3] = parse_set(sent, ld, d, lw, w, le, d->left, lcost, islands_ok, pi);
 				}
-				if (Rmatch) {
+				if (Rmatch)
+				{
 					rs[0] = parse_set(sent, d, rd, w, rw, d->right->next, re->next, rcost, islands_ok, pi);
 					if (d->right->multi) rs[1] = parse_set(sent, d, rd, w,rw,d->right,re->next, rcost, islands_ok, pi);
 					if (re->multi) rs[2] = parse_set(sent, d, rd, w, rw, d->right->next, re, rcost, islands_ok, pi);
 					if (d->right->multi && re->multi) rs[3] = parse_set(sent, d, rd, w, rw, d->right, re, rcost, islands_ok, pi);
 				}
 
-				for (i=0; i<4; i++) {
-					/* this ordering is probably not consistent with that needed to use list_links */
+				for (i=0; i<4; i++)
+				{
+					/* this ordering is probably not consistent with that
+					 *  needed to use list_links */
 					if (ls[i] == NULL) continue;
-					for (j=0; j<4; j++) {
+					for (j=0; j<4; j++)
+					{
 						if (rs[j] == NULL) continue;
 						a_choice = make_choice(ls[i], lw, w, le, d->left,
 											   rs[j], w, rw, d->right, re,
@@ -350,31 +372,42 @@ static Parse_set * parse_set(Sentence sent,
 					}
 				}
 
-				if (ls[0] != NULL || ls[1] != NULL || ls[2] != NULL || ls[3] != NULL) {
+				if (ls[0] != NULL || ls[1] != NULL || ls[2] != NULL || ls[3] != NULL)
+				{
 					/* evaluate using the left match, but not the right */
 					rset = parse_set(sent, d, rd, w, rw, d->right, re, rcost, islands_ok, pi);
-					if (rset != NULL) {
-						for (i=0; i<4; i++) {
+					if (rset != NULL)
+					{
+						for (i=0; i<4; i++)
+						{
 							if (ls[i] == NULL) continue;
-							/* this ordering is probably not consistent with that needed to use list_links */
+							/* this ordering is probably not consistent with
+							 * that needed to use list_links */
 							a_choice = make_choice(ls[i], lw, w, le, d->left,
-												   rset, w, rw, NULL /* d->right */, re,  /* the NULL indicates no link*/
-												   ld, d, rd);
+							         rset, w, rw, NULL /* d->right */,
+							         re,  /* the NULL indicates no link*/
+							         ld, d, rd);
 							put_choice_in_set(xt->set, a_choice);
 						}
 					}
 				}
-				if ((le == NULL) && (rs[0] != NULL || rs[1] != NULL || rs[2] != NULL || rs[3] != NULL)) {
+				if ((le == NULL) && (rs[0] != NULL ||
+				     rs[1] != NULL || rs[2] != NULL || rs[3] != NULL))
+				{
 					/* evaluate using the right match, but not the left */
 					lset = parse_set(sent, ld, d, lw, w, le, d->left, lcost, islands_ok, pi);
 
-					if (lset != NULL) {
-						for (i=0; i<4; i++) {
+					if (lset != NULL)
+					{
+						for (i=0; i<4; i++)
+						{
 							if (rs[i] == NULL) continue;
-							/* this ordering is probably not consistent with that needed to use list_links */
-							a_choice = make_choice(lset, lw, w, NULL /* le */, d->left,  /* NULL indicates no link */
-												   rs[i], w, rw, d->right, re,
-												   ld, d, rd);
+							/* this ordering is probably not consistent with
+							 * that needed to use list_links */
+							a_choice = make_choice(lset, lw, w, NULL /* le */,
+							          d->left,  /* NULL indicates no link */
+							          rs[i], w, rw, d->right, re,
+							          ld, d, rd);
 							put_choice_in_set(xt->set, a_choice);
 						}
 					}
@@ -397,26 +430,28 @@ static int verify_set_node(Parse_set *set)
 	s64 n;
 	if (set == NULL || set->first == NULL) return FALSE;
 	n = 0;
-	for (pc = set->first; pc != NULL; pc = pc->next) {
+	for (pc = set->first; pc != NULL; pc = pc->next)
+	{
 		n  += pc->set[0]->count * pc->set[1]->count;
-		if (PARSE_NUM_OVERFLOW < n) return 1;
+		if (PARSE_NUM_OVERFLOW < n) return TRUE;
 	}
-	return 0;
+	return FALSE;
 }
 
-static int verify_set(Parse_info pi) {
-	X_table_connector *t;
+static int verify_set(Parse_info pi)
+{
 	int i;
-	int overflowed;
 
-	overflowed=FALSE;
 	assert(pi->x_table != NULL, "called verify_set when x_table==NULL");
-	for (i=0; i<pi->x_table_size; i++) {
-		for(t = pi->x_table[i]; t!= NULL; t=t->next) {
-			overflowed = (overflowed || verify_set_node(t->set));
+	for (i=0; i<pi->x_table_size; i++)
+	{
+		X_table_connector *t;
+		for(t = pi->x_table[i]; t != NULL; t = t->next)
+		{
+			if (verify_set_node(t->set)) return TRUE;
 		}
 	}
-	return overflowed;
+	return FALSE;
 }
 
 /**
@@ -457,7 +492,7 @@ void free_parse_set(Sentence sent)
 	if (NULL == sent->parse_info) return;
 
 	/* This uses the x_table to free the whole parse set (the set itself
-	 * cannot be used cause it's a dag).  called from the outside world 
+	 * cannot be used cause it's a dag).  Called from the outside world.
 	 */
 	free_x_table(sent->parse_info);
 	sent->parse_info->parse_set = NULL;
